"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VisitInfo = void 0;
var graphql_compose_1 = require("graphql-compose");
var VisitInfo = /** @class */ (function () {
    function VisitInfo(data) {
        var _a;
        this.node = data.node;
        this.operation = data.operation;
        this.nodeParent = data.nodeParent;
        this.schemaComposer = data.schemaComposer;
        this.fieldPath = data.fieldPath;
        if (data.fieldName.indexOf('.')) {
            // if fieldName has dots, then split it
            var parts = data.fieldName.split('.').filter(Boolean);
            var fieldName = parts.pop();
            this.fieldName = fieldName;
            (_a = this.fieldPath).push.apply(_a, parts);
        }
        else {
            this.fieldName = data.fieldName;
        }
    }
    /**
     * Check that this entrypoint belongs to Query
     */
    VisitInfo.prototype.isQuery = function () {
        return this.operation === 'query';
    };
    /**
     * Check that this entrypoint belongs to Mutation
     */
    VisitInfo.prototype.isMutation = function () {
        return this.operation === 'mutation';
    };
    /**
     * Check that this entrypoint belongs to Subscription
     */
    VisitInfo.prototype.isSubscription = function () {
        return this.operation === 'subscription';
    };
    /**
     * Return array of fieldNames.
     * Dotted names will be automatically splitted.
     *
     * @example
     *   Assume:
     *     name: 'ping'
     *     path: ['query.storage', 'viewer', 'utils.debug']
     *   For empty options will be returned:
     *     ['storage', 'viewer', 'utils', 'debug', 'ping']
     *   For `{ includeOperation: true }` will be returned:
     *     ['query', 'storage', 'viewer', 'utils', 'debug', 'ping']
     */
    VisitInfo.prototype.getFieldPathArray = function (opts) {
        var res = [];
        this.fieldPath.forEach(function (e) {
            if (e.indexOf('.')) {
                res.push.apply(res, e.split('.').filter(Boolean));
            }
            else {
                res.push(e);
            }
        });
        if (!(opts === null || opts === void 0 ? void 0 : opts.omitFieldName)) {
            res.push(this.fieldName);
        }
        // slice(1) - remove first element from array
        return (opts === null || opts === void 0 ? void 0 : opts.includeOperation) ? res : res.slice(1);
    };
    /**
     * Return dotted path for current field
     */
    VisitInfo.prototype.getFieldPathDotted = function (opts) {
        return this.getFieldPathArray(opts).join('.');
    };
    /**
     * Return path as CamelCase string.
     *
     * Useful for getting type name according to path
     */
    VisitInfo.prototype.getFieldPathCamelCase = function (opts) {
        return this.getFieldPathArray(opts)
            .map(function (s) { return (0, graphql_compose_1.upperFirst)(s); })
            .join('');
    };
    Object.defineProperty(VisitInfo.prototype, "fieldConfig", {
        /**
         * Get FieldConfig for file or dir.
         * This is mutable object and is shared between all calls.
         */
        get: function () {
            var _a;
            var node = this.node;
            if (node.kind === 'file') {
                return node.fieldConfig;
            }
            else if (node.kind === 'dir' || this.node.kind === 'rootType') {
                // TODO: think about namespaceConfig (how to do it not null)
                return (_a = node.namespaceConfig) === null || _a === void 0 ? void 0 : _a.fieldConfig;
            }
            throw new Error("Cannot get fieldConfig. Node has some strange kind: ".concat(node.kind));
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Get TypeComposer instance for output type (object, scalar, enum, interface, union).
     * It's mutable object.
     */
    VisitInfo.prototype.getOutputAnyTC = function () {
        var _a;
        var fc = this.fieldConfig;
        var outputType = fc.type;
        if (!outputType) {
            throw new Error("FieldConfig ".concat(this.getFieldPathDotted(), " does not have 'type' property"));
        }
        // if the type is of any kind of TypeComposer
        // then return it directly
        // or try to convert it to TypeComposer and save in FieldConfig as prepared type
        if ((0, graphql_compose_1.isTypeComposer)(outputType)) {
            return outputType;
        }
        else {
            var outputTC = this.schemaComposer.typeMapper.convertOutputTypeDefinition(outputType, this.fieldName, (_a = this.nodeParent) === null || _a === void 0 ? void 0 : _a.name);
            if (!outputTC) {
                throw new Error("FieldConfig ".concat(this.getFieldPathDotted(), " contains some strange value as output type"));
            }
            fc.type = outputTC;
            return outputTC;
        }
    };
    /**
     * Check that output type is an object
     */
    VisitInfo.prototype.isOutputTypeIsObject = function () {
        return this.getOutputAnyTC() instanceof graphql_compose_1.ObjectTypeComposer;
    };
    /**
     * Get TypeComposer instance for output type (object, scalar, enum, interface, union).
     * It's mutable object.
     */
    VisitInfo.prototype.getOutputUnwrappedTC = function () {
        return (0, graphql_compose_1.unwrapOutputTC)(this.getOutputAnyTC());
    };
    /**
     * Get TypeComposer instance for output type (object, scalar, enum, interface, union).
     * It's mutable object.
     */
    VisitInfo.prototype.getOutputUnwrappedOTC = function () {
        var tc = (0, graphql_compose_1.unwrapOutputTC)(this.getOutputAnyTC());
        if (!(tc instanceof graphql_compose_1.ObjectTypeComposer)) {
            throw new Error("FieldConfig ".concat(this.getFieldPathDotted(), " has non-Object output type. Use 'isOutputTypeIsObject()' before for avoiding this error."));
        }
        return tc;
    };
    VisitInfo.prototype.toString = function () {
        return "VisitInfo(".concat(this.getFieldPathDotted({ includeOperation: true }), ")");
    };
    VisitInfo.prototype.toJSON = function () {
        return this.toString();
    };
    return VisitInfo;
}());
exports.VisitInfo = VisitInfo;
//# sourceMappingURL=VisitInfo.js.map