"use strict";
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFields = exports.astToSchema = void 0;
var graphql_compose_1 = require("graphql-compose");
var dedent_1 = __importDefault(require("dedent"));
var graphql_1 = require("graphql");
/**
 * Transform AST to GraphQL Schema.
 *
 * @example
 *   // Scan some directory for getting Schema entrypoints as AST nodes.
 *   const ast = directoryToAst(module);
 *
 *   // [Optional] Combining severals ast into source
 *   // Useful if some sub-schemas are delivered via packages
 *   // or created n separate directories.
 *   const newAST = astMerge(ast, ast1, ast2, ...);
 *
 *   // [Optional] Some `ast` modifications
 *   // Useful for writing some middlewares
 *   // which transform FieldConfigs entrypoints.
 *   astVisitor(newAST, visitorFns);
 *
 *   // Create SchemaComposer instance with all populated types & fields
 *   // It provides declarative programmatic access to modify you schema
 *   const schemaComposer = astToSchema(newAST, opts);
 *
 *   // Create GraphQLSchema instance which is ready for runtime.
 *   const schema = schemaComposer.buildSchema();;
 */
function astToSchema(ast, opts) {
    if (opts === void 0) { opts = {}; }
    var sc;
    if (opts === null || opts === void 0 ? void 0 : opts.schemaComposer) {
        if (!opts.schemaComposer) {
            throw new Error((0, dedent_1.default)(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n        Provided option 'schemaComposer' should be an instance of SchemaComposer class from 'graphql-compose' package.\n        Received:\n          ", "\n      "], ["\n        Provided option 'schemaComposer' should be an instance of SchemaComposer class from 'graphql-compose' package.\n        Received:\n          ", "\n      "])), (0, graphql_compose_1.inspect)(opts.schemaComposer)));
        }
        sc = opts.schemaComposer;
    }
    else {
        sc = new graphql_compose_1.SchemaComposer();
    }
    if (ast.children.query)
        populateRoot(sc, 'Query', ast.children.query, opts);
    if (ast.children.mutation)
        populateRoot(sc, 'Mutation', ast.children.mutation, opts);
    if (ast.children.subscription)
        populateRoot(sc, 'Subscription', ast.children.subscription, opts);
    return sc;
}
exports.astToSchema = astToSchema;
function populateRoot(sc, rootName, astRootNode, opts) {
    var tc = sc[rootName];
    Object.keys(astRootNode.children).forEach(function (key) {
        createFields(sc, astRootNode.children[key], tc, rootName, opts || {});
    });
}
function createFields(sc, ast, parent, pathPrefix, opts) {
    var _a, _b;
    if (opts === void 0) { opts = {}; }
    if (!ast)
        return;
    var name = ast.name;
    if (!/^[._a-zA-Z0-9]+$/.test(name)) {
        throw new Error("You provide incorrect ".concat(ast.kind === 'file' ? 'file' : 'directory', " name '").concat(name, "', it should meet RegExp(/^[._a-zA-Z0-9]+$/) for '").concat(ast.absPath, "'"));
    }
    if (ast.kind === 'file') {
        parent.addNestedFields((_a = {},
            _a[name] = ast.fieldConfig,
            _a));
        return;
    }
    if (ast.kind === 'dir') {
        var typename = getTypename(ast, pathPrefix, opts);
        var fc_1;
        if (ast.namespaceConfig) {
            fc_1 = prepareNamespaceFieldConfig(sc, ast.namespaceConfig, typename);
        }
        else {
            fc_1 = { type: sc.createObjectTC(typename) };
        }
        parent.addNestedFields((_b = {},
            _b[name] = __assign({ resolve: function (source) { return source; } }, fc_1),
            _b));
        var pathPrefixForChild_1 = getTypename(ast, pathPrefix, {});
        Object.keys(ast.children).forEach(function (key) {
            createFields(sc, ast.children[key], fc_1.type, pathPrefixForChild_1, opts);
        });
    }
}
exports.createFields = createFields;
function getTypename(ast, pathPrefix, opts) {
    var name = ast.name;
    var typename = pathPrefix;
    if (name.indexOf('.') !== -1) {
        var namesArray = name.split('.');
        if (namesArray.some(function (n) { return !n; })) {
            throw new Error("Field name '".concat(ast.name, "' contains dots in the wrong place for '").concat(ast.absPath, "'!"));
        }
        typename += namesArray.reduce(function (prev, current) {
            return prev + (0, graphql_compose_1.upperFirst)(current);
        }, '');
    }
    else {
        typename += (0, graphql_compose_1.upperFirst)(name);
    }
    if (opts.prefix)
        typename = "".concat(opts.prefix).concat(typename);
    if (opts.suffix)
        typename += opts.suffix;
    return typename;
}
function prepareNamespaceFieldConfig(sc, ast, typename) {
    var fc = ast.fieldConfig;
    if (!fc.type) {
        fc.type = sc.createObjectTC(typename);
    }
    else {
        if (typeof fc.type === 'string') {
            if (!(0, graphql_compose_1.isOutputTypeDefinitionString)(fc.type) && !(0, graphql_compose_1.isWrappedTypeNameString)(fc.type)) {
                throw new Error((0, dedent_1.default)(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n          You provide incorrect output type definition:\n            ", "\n          It must be valid TypeName or output type SDL definition:\n\n          Eg.\n            type Payload { me: String }\n          OR\n            Payload\n        "], ["\n          You provide incorrect output type definition:\n            ", "\n          It must be valid TypeName or output type SDL definition:\n\n          Eg.\n            type Payload { me: String }\n          OR\n            Payload\n        "])), fc.type));
            }
        }
        else if (!(fc.type instanceof graphql_compose_1.ObjectTypeComposer) &&
            !(fc.type instanceof graphql_1.GraphQLObjectType)) {
            throw new Error((0, dedent_1.default)(templateObject_3 || (templateObject_3 = __makeTemplateObject(["\n        You provide some strange value as 'type':\n          ", "\n      "], ["\n        You provide some strange value as 'type':\n          ", "\n      "])), (0, graphql_compose_1.inspect)(fc.type)));
        }
        fc.type = sc.createObjectTC(fc.type);
    }
    if (!fc.resolve) {
        fc.resolve = function () { return ({}); };
    }
    return fc;
}
var templateObject_1, templateObject_2, templateObject_3;
//# sourceMappingURL=astToSchema.js.map