"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.forEachKey = exports.visitNode = exports.astVisitor = exports.VISITOR_REMOVE_NODE = exports.VISITOR_SKIP_CHILDREN = void 0;
var VisitInfo_1 = require("./VisitInfo");
/**
 * Do not traverse children, and go to next sibling.
 */
exports.VISITOR_SKIP_CHILDREN = false;
/**
 * Remove Node from AST and do not traverse children.
 */
exports.VISITOR_REMOVE_NODE = null;
/**
 * Traverse AST for applying modifications to DIRs, FILEs & ROOT_TYPEs
 * Useful for writing middlewares which transform FieldConfigs entrypoints.
 *
 * @example
 *   const ast = directoryToAst(module);
 *   astVisitor(ast, schemaComposer, {
 *     ROOT_TYPE: () => {}, // run for query, mutation, subscription
 *     DIR: () => {}, // executes on visiting DIR node
 *     FILE: () => {}, // executes on visiting FILE node
 *   });
 */
function astVisitor(ast, schemaComposer, visitor) {
    Object.keys(ast.children).forEach(function (operation) {
        var node = ast.children[operation];
        if (!node)
            return;
        visitNode(new VisitInfo_1.VisitInfo({
            node: node,
            nodeParent: ast,
            fieldName: operation,
            fieldPath: [],
            operation: operation,
            schemaComposer: schemaComposer,
        }), visitor);
    });
}
exports.astVisitor = astVisitor;
function visitNode(info, visitor) {
    var result;
    if (info.node.kind === 'dir') {
        if (visitor.DIR)
            result = visitor.DIR(info);
    }
    else if (info.node.kind === 'file') {
        if (visitor.FILE)
            result = visitor.FILE(info);
    }
    else if (info.node.kind === 'rootType') {
        if (visitor.ROOT_TYPE)
            result = visitor.ROOT_TYPE(info);
    }
    if (result === exports.VISITOR_REMOVE_NODE) {
        // `null` - means remove node from Ast and do not traverse children
        delete info.nodeParent.children[info.node.name];
        return;
    }
    else if (result === exports.VISITOR_SKIP_CHILDREN) {
        // `false` - do not traverse children
        return;
    }
    else if (result) {
        // replace node
        info.nodeParent.children[info.node.name] = result;
    }
    else {
        // `undefined` - just move further
        result = info.node;
    }
    if (result.kind === 'dir' || result.kind === 'rootType') {
        forEachKey(result.children, function (childNode, fieldName) {
            visitNode(new VisitInfo_1.VisitInfo({
                node: childNode,
                nodeParent: result,
                fieldName: fieldName,
                fieldPath: __spreadArray(__spreadArray([], info.fieldPath, true), [info.fieldName], false),
                operation: info.operation,
                schemaComposer: info.schemaComposer,
            }), visitor);
        });
    }
}
exports.visitNode = visitNode;
function forEachKey(obj, callback) {
    Object.keys(obj).forEach(function (key) {
        callback(obj[key], key);
    });
}
exports.forEachKey = forEachKey;
//# sourceMappingURL=astVisitor.js.map