"use strict";
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAstForFile = exports.getAstForDir = exports.directoryToAst = exports.defaultOptions = void 0;
var fs_1 = __importDefault(require("fs"));
var graphql_compose_1 = require("graphql-compose");
var path_1 = require("path");
exports.defaultOptions = {
    extensions: ['js', 'ts'],
};
/**
 * Traverses directories and construct AST for your graphql entrypoints.
 *
 * @param m – is a NodeJS Module which provides a way to load modules from scanned dir in the regular nodejs way
 * @param options – set of options which helps to customize rules of what files/dirs should be loaded or not
 */
function directoryToAst(m, options) {
    if (options === void 0) { options = exports.defaultOptions; }
    // @ts-ignore
    if (options === null || options === void 0 ? void 0 : options.relativePath) {
        throw new Error('graphql-compose-modules: `relativePath` option is deprecated use `rootDir` instead');
    }
    // if no path was passed in, assume the equivalent of __dirname from caller
    // otherwise, resolve path relative to the equivalent of __dirname
    var schemaPath = (options === null || options === void 0 ? void 0 : options.rootDir)
        ? (0, path_1.resolve)((0, path_1.dirname)(m.filename), options.rootDir)
        : (0, path_1.dirname)(m.filename);
    // setup default options
    Object.keys(exports.defaultOptions).forEach(function (prop) {
        if (typeof options[prop] === 'undefined') {
            options[prop] = exports.defaultOptions[prop];
        }
    });
    var result = {
        kind: 'root',
        name: (0, path_1.basename)(schemaPath),
        absPath: schemaPath,
        children: {},
    };
    fs_1.default.readdirSync(schemaPath).forEach(function (filename) {
        var absPath = (0, path_1.join)(schemaPath, filename);
        if (fs_1.default.statSync(absPath).isDirectory()) {
            var dirName = filename;
            var re = /^(query|mutation|subscription)(\.(.*))?$/i;
            var found = dirName.match(re);
            if (found) {
                var opType = found[1].toLowerCase();
                var rootTypeAst = result.children[opType];
                if (!rootTypeAst)
                    rootTypeAst = {
                        kind: 'rootType',
                        name: opType,
                        absPath: absPath,
                        children: {},
                    };
                var astDir = getAstForDir(m, absPath, options);
                if (astDir) {
                    var subField = found[3]; // any part after dot (eg for `query.me` will be `me`)
                    if (subField) {
                        rootTypeAst.children[subField] = __assign(__assign({}, astDir), { name: subField, absPath: absPath });
                    }
                    else {
                        rootTypeAst.children = astDir.children;
                        if (astDir.namespaceConfig) {
                            rootTypeAst.namespaceConfig = astDir.namespaceConfig;
                        }
                    }
                    result.children[opType] = rootTypeAst;
                }
            }
        }
    });
    return result;
}
exports.directoryToAst = directoryToAst;
function getAstForDir(m, absPath, options) {
    if (options === void 0) { options = exports.defaultOptions; }
    var name = (0, path_1.basename)(absPath);
    if (!checkInclusion(absPath, 'dir', name, options))
        return;
    var result = {
        kind: 'dir',
        absPath: absPath,
        name: name,
        children: {},
    };
    // get the path of each file in specified directory, append to current tree node, recurse
    fs_1.default.readdirSync(absPath).forEach(function (filename) {
        var absFilePath = (0, path_1.join)(absPath, filename);
        var stat = fs_1.default.statSync(absFilePath);
        if (stat.isDirectory()) {
            // this node is a directory; recurse
            if (result.children[filename]) {
                throw new Error("You have a folder and file with same name \"".concat(filename, "\" by the following path ").concat(absPath, ". Please remove one of them."));
            }
            var astDir = getAstForDir(m, absFilePath, options);
            if (astDir) {
                result.children[filename] = astDir;
            }
        }
        else if (stat.isFile()) {
            // this node is a file
            var fileAst = getAstForFile(m, absFilePath, options);
            if (fileAst) {
                if (fileAst.name === 'index') {
                    result.namespaceConfig = fileAst;
                }
                else if (result.children[fileAst.name]) {
                    throw new Error("You have a folder and file with same name \"".concat(fileAst.name, "\" by the following path ").concat(absPath, ". Please remove one of them."));
                }
                else {
                    result.children[fileAst.name] = fileAst;
                }
            }
        }
    });
    return result;
}
exports.getAstForDir = getAstForDir;
function getAstForFile(m, absPath, options) {
    if (options === void 0) { options = exports.defaultOptions; }
    var filename = (0, path_1.basename)(absPath);
    if (absPath !== m.filename && checkInclusion(absPath, 'file', filename, options)) {
        // module name shouldn't include file extension
        var moduleName = filename.substring(0, filename.lastIndexOf('.'));
        // namespace configs may not have `type` property
        var checkType = moduleName !== 'index';
        var code = m.require(absPath);
        var fieldConfig = prepareFieldConfig(code, absPath, checkType);
        return {
            kind: 'file',
            name: moduleName,
            absPath: absPath,
            code: code,
            fieldConfig: fieldConfig,
        };
    }
}
exports.getAstForFile = getAstForFile;
function checkInclusion(absPath, kind, filename, options) {
    // Skip dir/files started from double underscore
    if (/^__.*/i.test(filename)) {
        return false;
    }
    // Skip dir/files started from dot
    if (/^\..*/i.test(filename)) {
        return false;
    }
    if (kind === 'file') {
        if (
        // Verify file has valid extension
        !new RegExp('\\.(' + ((options === null || options === void 0 ? void 0 : options.extensions) || ['js', 'ts']).join('|') + ')$', 'i').test(filename) ||
            // Hardcoded skip file extensions
            // typescript definition files
            new RegExp('(\\.d\\.ts)$', 'i').test(filename) ||
            // test files
            new RegExp('(\\.spec\\.(ts|js))$', 'i').test(filename)) {
            return false;
        }
    }
    if (options.include) {
        if (options.include instanceof RegExp) {
            // if options.include is a RegExp, evaluate it and make sure the path passes
            if (!options.include.test(absPath))
                return false;
        }
        else if (typeof options.include === 'function') {
            // if options.include is a function, evaluate it and make sure the path passes
            if (!options.include(absPath, kind, filename))
                return false;
        }
    }
    if (options.exclude) {
        if (options.exclude instanceof RegExp) {
            // if options.exclude is a RegExp, evaluate it and make sure the path doesn't pass
            if (options.exclude.test(absPath))
                return false;
        }
        else if (typeof options.exclude === 'function') {
            // if options.exclude is a function, evaluate it and make sure the path doesn't pass
            if (options.exclude(absPath, kind, filename))
                return false;
        }
    }
    return true;
}
function prepareFieldConfig(code, absPath, checkType) {
    if (checkType === void 0) { checkType = true; }
    var _fc = code === null || code === void 0 ? void 0 : code.default;
    if (!_fc || typeof _fc !== 'object') {
        throw new Error((0, graphql_compose_1.dedent)(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n      GraphQL entrypoint MUST return FieldConfig as default export in '", "'. \n      Eg:\n        export default {\n          type: 'String',\n          resolve: () => Date.now(),\n        };\n    "], ["\n      GraphQL entrypoint MUST return FieldConfig as default export in '", "'. \n      Eg:\n        export default {\n          type: 'String',\n          resolve: () => Date.now(),\n        };\n    "])), absPath));
    }
    var fc;
    if (code.default instanceof graphql_compose_1.Resolver) {
        fc = code.default.getFieldConfig();
    }
    else {
        // recreate object for immutability purposes (do not change object in module definition)
        // NB. I don't know should we here recreate (args, extensions) but let's keep them as is for now.
        fc = __assign({}, code.default);
    }
    if (checkType) {
        if (!fc.type || !isSomeOutputTypeDefinition(fc.type)) {
            throw new Error((0, graphql_compose_1.dedent)(templateObject_2 || (templateObject_2 = __makeTemplateObject(["\n      Module MUST return FieldConfig with correct 'type: xxx' property in '", "'. \n      Eg:\n        export default {\n          type: 'String'\n        };\n    "], ["\n      Module MUST return FieldConfig with correct 'type: xxx' property in '", "'. \n      Eg:\n        export default {\n          type: 'String'\n        };\n    "])), absPath));
        }
    }
    if (fc.resolve && typeof fc.resolve !== 'function') {
        throw new Error("Cannot load entrypoint config from ".concat(absPath, ". 'resolve' property must be a function or undefined."));
    }
    return fc;
}
function isSomeOutputTypeDefinition(type) {
    if (typeof type === 'string') {
        // type: 'String'
        return (0, graphql_compose_1.isSomeOutputTypeDefinitionString)(type) || (0, graphql_compose_1.isWrappedTypeNameString)(type);
    }
    else if (Array.isArray(type)) {
        // type: ['String']
        return isSomeOutputTypeDefinition(type[0]);
    }
    else if ((0, graphql_compose_1.isFunction)(type)) {
        // pass thunked type without internal checks
        return true;
    }
    else {
        // type: 'type User { name: String }'
        return (0, graphql_compose_1.isComposeOutputType)(type);
    }
}
var templateObject_1, templateObject_2;
//# sourceMappingURL=directoryToAst.js.map